/*
 *   Compute the following regarding an object:
 *
 *     - Total area
 *     - Center of mass
 *     - Moments of inertia
 *     - Major axis orientation
 *     - eccentricity
 *
 *     This is from p. 255, Computer Vision, Ballard and Brown.
 *     Note that we're doing this over the whole object, not just the
 *     perimeter.
 */

#include <stdio.h>
#include <sys/types.h>
#include <math.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#define PI    3.141592653589
#define FALSE 0
#define TRUE 1
#define DIM 50             /* Image is DIMxDIM (was 10). */
#define N 100              /* Max number of units allowed in net */

/*  X stuff */

Display *disptr;
GC       gc;

/*  Object specification */

char Objfname[80];
char realObjfname[80];
Pixmap Objpmap;
XImage *Objimage, *Newimage;
int Width, Height, x_hot, y_hot;

/*  Neat stuff calculated regarding this object */

int Area;
int MeanX, MeanY;         /* CoM = center of mass */
double Moments[3][3];        /* Moments of inertia */
double Theta;                /* MAO = major axis orientation */
double Eccentricity;         /* "Elongatedness" */

main(argc,argv)
     int argc;
     char *argv[];
{
  int ing;                  /* Needed for command line parsing */
  extern char *optarg;      /* Needed for command line parsing */

  int *pointertopmap;
  double f;
  char inc, OutputObjfname[80];

  int bitmap_size;
  int status, x,y, i,j;

  Objfname[0] = '*';

  disptr = XOpenDisplay(NULL);
  gc = DefaultGC(disptr, 0);

  /* Read in options */

  while ((ing = getopt(argc,argv,"o:")) != EOF)
    switch (ing)
      {
      case 'o': strcpy(Objfname, optarg);    break;
      }

  /* Read in the object (will be in the Objects/ directory). */

  if (Objfname[0] == '*')
    {
      printf (">> Error:  No object specified.  Use -o <Object>.\n");
      exit(1);
    }
  else
    {
      sprintf (realObjfname, "Filled%s", Objfname);
      status = XReadBitmapFile(disptr,XDefaultRootWindow(disptr),
			       realObjfname, &Width, &Height, &Objpmap, 
			       &x_hot, &y_hot);
      if (status != BitmapSuccess)
	{
	  printf (">> Error:  Could not read object file %s.\n", 
		  realObjfname);
	  exit(1);
	}
    }

  /*
   *  Get the object, then calculate mean vector, moments, 
   *    theta, and eccentricity.   Etc.  And output. 
   */

  Objimage = XGetImage(disptr,Objpmap, 0,0, Width,Height, 
		       1, XYPixmap);
  MomentsEtc();
}


/*
 *  Calculate moments M[i][j] and all that other stuff.
 */
MomentsEtc()
{
  register int i,j, x,y;
  int sumx, sumy;
  double tmpsin,tmpcos;
  int a, b, c;             /* Second moments from Horn, p. 53 */
  double da,db,dc;         /* Double versions of above. */
  int xdiff,ydiff, characteristic;
  double minorTheta, minErr,maxErr;

  /* Center of mass first */

  Area = 0;   
  sumx = 0; 
  sumy=0;

  for (i=0; i<DIM; i++)
    for (j=0; j<DIM; j++)
      if (XGetPixel(Objimage, i,j) == 1)     /* Part of interior */
	{
	  Area++;
	  sumx += i;
	  sumy += j;
	}

  MeanX = (int) ((double) sumx / (double) Area);
  MeanY = (int) ((double) sumy / (double) Area);


  /* Formula from Horn, p. 53:
   *
   *    tan (2*theta) = b / (a-c).  
   */

  a = 0;
  b = 0;
  c = 0;

  for (x=0; x<DIM; x++)
    for (y=0; y<DIM; y++)
      {
	xdiff = x - MeanX;
	ydiff = y - MeanY;
	characteristic = XGetPixel(Objimage, x,y);

	a = a + (xdiff * xdiff * characteristic);
	b = b + (2 * xdiff * ydiff * characteristic);
	c = c + (ydiff * ydiff * characteristic);
      }


  /*  Results */

  printf ("These are vital statistics on object %s.\n", Objfname);
  printf ("Angles are represented by sin,cos.\n#\n");

  printf ("Area:  %d\nCoM:  %d %d\n", Area, MeanX,MeanY);

  if ((b == 0) &&             /* Object is symmetrical => Ecc == 0  */
      (a == c))
    {
      printf ("Eccentricity:  0.0\n");
      printf ("MAO:  0.000 0.000\nmAO:  0.000 0.000\n");
    }

  else
    {
      da = (double)a;
      db = (double)b;
      dc = (double)c;

      /*  
       *  Note:
       *      Atan2 returns the arc tangent of y/x in the range -J to J.
       *      double atan2(y, x)
       *          double x, y;
       */

      Theta = 0.5 * atan2(db, da-dc);
      minorTheta = Theta + (PI/2.0);

      minErr = 
	(0.5*(da+dc)) - 
	  (0.5*(da-dc)*cos(2.0*Theta)) -
	    (0.5*db*sin(2.0*Theta));

      maxErr = 
	(0.5*(da+dc)) - 
	  (0.5*(da-dc)*cos(2.0*minorTheta)) -
	    (0.5*db*sin(2.0*minorTheta));

      Eccentricity = 1.0 - (minErr / maxErr);

      printf ("Eccentricity:  %f\n", Eccentricity);
      printf ("MAO:  %f %f\nmAO:  %f %f\n", 
	      sin(Theta),cos(Theta), sin(minorTheta),cos(minorTheta));

    }
}


